/*******************************************************************************
* FILE NAME: user_routines.c
*
* DESCRIPTION:
*  Initialization code and regular driving control.  
*
* ASSUMPTIONS:
*   rc_dig_out05 - The Sonic pulse input line (trigger pulse)
*   rc_dig_int1 - Sonic output line (echo pulse)
*   rc_dig_out15 - LED is on when the robot is at the distance set by the pot below      
*   rc_ana_in04 - Onboard Potentiometer input to control distance       
*   rc_dig_in06 - ONBOARD SWITCH to go into autonomous mode
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "stdio.h"
#include "Ultrasonic_sensor.h" //Sonic


static void Setup_Who_Controls_Pwms(int pwmSpec1,int pwmSpec2,int pwmSpec3,int pwmSpec4,
                                    int pwmSpec5,int pwmSpec6,int pwmSpec7,int pwmSpec8)
{
  txdata.pwm_mask = 0xFF;         /* Default to master controlling all PWMs. */
  if (pwmSpec1 == USER)           /* If User controls PWM1 then clear bit0. */
    txdata.pwm_mask &= 0xFE;      /* same as txdata.pwm_mask = txdata.pwm_mask & 0xFE; */
  if (pwmSpec2 == USER)           /* If User controls PWM2 then clear bit1. */
    txdata.pwm_mask &= 0xFD;
  if (pwmSpec3 == USER)           /* If User controls PWM3 then clear bit2. */
    txdata.pwm_mask &= 0xFB;
  if (pwmSpec4 == USER)           /* If User controls PWM4 then clear bit3. */
    txdata.pwm_mask &= 0xF7;
  if (pwmSpec5 == USER)           /* If User controls PWM5 then clear bit4. */
    txdata.pwm_mask &= 0xEF;
  if (pwmSpec6 == USER)           /* If User controls PWM6 then clear bit5. */
    txdata.pwm_mask &= 0xDF;
  if (pwmSpec7 == USER)           /* If User controls PWM7 then clear bit6. */
    txdata.pwm_mask &= 0xBF;
  if (pwmSpec8 == USER)           /* If User controls PWM8 then clear bit7. */
    txdata.pwm_mask &= 0x7F;
}

void User_Initialization (void)
{
  IO1 = IO2 = IO3 = IO4 = IO5 = IO6 = INPUT;        /* Used for analog inputs. */

  Set_Number_of_Analog_Channels(SIX_ANALOG);     /* See ifi_aliases.h */

  IO8 = IO10 = INPUT;     
  IO12 = INPUT;        

  IO7 = IO9 = OUTPUT;     
  IO11 = IO13 = IO14 = IO15 = IO16 = OUTPUT;   

  rc_dig_out07 = 0;
  rc_dig_out09 = rc_dig_out11 = 0;
  rc_dig_out13 = rc_dig_out14 = rc_dig_out15 = rc_dig_out16 = 0;

  pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;

  Setup_Who_Controls_Pwms(MASTER,MASTER,MASTER,MASTER,MASTER,MASTER,MASTER,MASTER);
  Setup_PWM_Output_Type(IFI_PWM,IFI_PWM,IFI_PWM,IFI_PWM);

  Ultrasonic_Initialization();  //Sonic
  Initialize_Serial_Comms();   

  txdata.user_cmd = 0x02;       /* Tell master you want to be in auton mode (Receivers disabled) */

  Putdata(&txdata);             /* DO NOT CHANGE! */
  User_Proc_Is_Ready();         /* DO NOT CHANGE! - last line of User_Initialization */

#ifdef _SIMULATOR
  statusflag.NEW_SPI_DATA = 1;
#else
  /* This code receives the 1st packet from master to obtain the version # */
  while (!statusflag.NEW_SPI_DATA);  /* Wait for 1st packet from master */
  Getdata(&rxdata);   
  printf("VEX - Master v%d\r",(int)rxdata.master_version);        
#endif
}


void Process_Data_From_Master_uP(void)
{
  Getdata(&rxdata);   /* Get fresh data from the master microprocessor. */

  Default_Routine();  /* Optional.  See below. */

  /* Add your own code here. */

  Ultra_Sensor();  //Sonic - Keep track of any walls in front of us

  if (ONBOARD_SWITCH == 0)
  {
    txdata.user_cmd |= 0x02;  // Tell Master we want to be in autonomous mode
  }
  else
  {
    txdata.user_cmd = 0; // We don't want to be in autonomous mode
  }

  Putdata(&txdata);             /* DO NOT CHANGE! */
}

/*******************************************************************************
* FUNCTION NAME: Default_Routine
* PURPOSE:       Performs the default mappings of inputs to outputs for  
*                CONFIGURATIONS A, B, and C (B = drive mode 12 - settings on radio)
* CALLED FROM:   this file, Process_Data_From_Master_uP routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
#define BUTTON_REV_THRESH       100
#define BUTTON_FWD_THRESH       154
#define NEUTRAL_VALUE           127

void Default_Routine(void)
{           
  unsigned int pot;
  int temp;

  //Command the maximum range (joystick doesn't go all the way)
  if (PWM_in1 > 200)PWM_in1=254;
  if (PWM_in1 < 40)PWM_in1=0;
  if (PWM_in2 > 200)PWM_in2=254;
  if (PWM_in2 < 40)PWM_in2=0;

  pwm01 = PWM_in1;
  pwm02 = PWM_in2;
  pwm03 = 255 - PWM_in3;      // reverse direction of left side (CCW)  
  pwm04 = PWM_in4;

  // Match the motor power on both sides
  temp = (int)pwm02 - 127;
  if (pwm02 < 127) pwm02 -= FORWARDBALANCE*temp/127;
  temp = (int)pwm01 - 127;
  if (pwm01 < 127) pwm01 -= BACKWARDBALANCE*temp/127;


  pwm05 = 0x7F;               //Handle Channel 5 receiver button
  if (PWM_in5 < BUTTON_REV_THRESH)
  {
    pwm05 = 0xFF;
  }
  else if (PWM_in5 > BUTTON_FWD_THRESH)
  {
    pwm05 = 0;
  }
  pwm06 = 0x7F;               //Handle Channel 6 receiver button
  if (PWM_in6 < BUTTON_REV_THRESH)
    pwm06 = 0xFF;
  else if (PWM_in6 > BUTTON_FWD_THRESH)
    pwm06 = 0;


} /* END Default_Routine(); */


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
